@@ -20,6 +20,19 @@ class IdentityLogger(torch.nn.Module):
         return self.tensors[-1]
 
 
+def test_gradient_attributor_inactive(
+    data_linear, model_simple, model_simple_output, any_composite, grad_outputs_func
+):
+    '''Test whether composite context and attributor match for Gradient.'''
+
+    with Gradient(model=model_simple, composite=any_composite, attr_output=grad_outputs_func) as attributor:
+        # verify that all hooks are active
+        assert all(hook.active for hook in attributor.composite.hook_refs)
+        with attributor.inactive():
+            # verify that all hooks are inactive
+            assert all(not hook.active for hook in attributor.composite.hook_refs)
+
+
 def test_gradient_attributor_composite(
     data_linear, model_simple, model_simple_output, any_composite, grad_outputs_func
 ):
@@ -61,8 +74,21 @@ def test_gradient_attributor_output_fn(data_simple, grad_outputs_func, use_const
     assert torch.allclose(expected_grad, grad), 'Attributor output function gradient mismatch!'
 
 
+def test_gradient_attributor_grad(data_simple):
+    '''Test whether the gradient of Gradient matches.'''
+    model = torch.nn.Softplus(beta=1.)
+    data = data_simple.view_as(data_simple).requires_grad_()
+    target = torch.sigmoid(data) * (1 - torch.sigmoid(data))
+
+    with Gradient(model=model, create_graph=True) as attributor:
+        _, grad = attributor(data, torch.ones_like)
+    gradgrad, = torch.autograd.grad(grad.sum(), data)
+
+    assert torch.allclose(gradgrad, target), 'Gradient Attributor second order gradient mismatch!'
+
+
 def test_gradient_attributor_output_fn_precedence(data_simple):
-    '''Test whether the gradient attributor attr_output at call is prefered when it is supplied at both initialization
+    '''Test whether the gradient attributor attr_output at call is preferred when it is supplied at both initialization
     and call.
     '''
     model = IdentityLogger()
@@ -86,6 +112,19 @@ def test_smooth_grad_single(data_linear, model_simple, model_simple_output, mode
     assert torch.allclose(model_simple_output, output)
 
 
+def test_smooth_grad_single_grad(data_simple):
+    '''Test whether the gradient of SmoothGrad matches.'''
+    model = torch.nn.Softplus(beta=1.)
+    data = data_simple.view_as(data_simple).requires_grad_()
+    target = torch.sigmoid(data) * (1 - torch.sigmoid(data))
+
+    with SmoothGrad(model=model, noise_level=0.1, n_iter=1, create_graph=True) as attributor:
+        _, grad = attributor(data, torch.ones_like)
+    gradgrad, = torch.autograd.grad(grad.sum(), data)
+
+    assert torch.allclose(gradgrad, target), 'SmoothGrad Attributor second order gradient mismatch!'
+
+
 @pytest.mark.parametrize('noise_level', [0.0, 0.1, 0.3, 0.5])
 def test_smooth_grad_distribution(data_simple, noise_level):
     '''Test whether the SmoothGrad sampled distribution matches.'''
@@ -98,12 +137,17 @@ def test_smooth_grad_distribution(data_simple, noise_level):
     with SmoothGrad(model=model, noise_level=noise_level, n_iter=n_iter, attr_output=torch.ones_like) as attributor:
         _, grad = attributor(data_simple)
 
-    assert len(model.tensors) == n_iter, 'SmootGrad iterations did not match n_iter!'
+    assert len(model.tensors) == n_iter, 'SmoothGrad iterations did not match n_iter!'
 
     sample_mean = sum(model.tensors) / len(model.tensors)
     sample_var = ((sum((tensor - sample_mean) ** 2 for tensor in model.tensors) / len(model.tensors))).mean(dims)
 
-    assert torch.allclose(sample_var, noise_var, rtol=0.2), 'SmoothGrad sample variance is too high!'
+    if noise_level > 0.:
+        std_ratio = (sample_var / noise_var) ** .5
+        assert (std_ratio < 1.5).all().item(), 'SmoothGrad sample variance is too high!'
+        assert (std_ratio > 0.667).all().item(), 'SmoothGrad sample variance is too low!'
+    else:
+        assert (sample_var < 1e-9).all().item(), 'SmoothGrad sample variance is too high!'
     assert torch.allclose(grad, torch.ones_like(data_simple)), 'SmoothGrad of identity is wrong!'
 
 
@@ -123,6 +167,20 @@ def test_integrated_gradients_single(data_linear, model_simple, model_simple_out
     assert torch.allclose(model_simple_output, output), 'Output mismatch for IntegratedGradients!'
 
 
+def test_integrated_gradients_single_grad(data_simple):
+    '''Test whether the gradient of IntegratedGradients matches.'''
+    model = torch.nn.Softplus(beta=1.)
+    data = data_simple.view_as(data_simple).requires_grad_()
+    # this is d/dx (x * d/dx softplus(x)), i.e. the gradient of input times gradient of softplus
+    target = torch.sigmoid(data) * (1 - torch.sigmoid(data)) * data + torch.sigmoid(data)
+
+    with IntegratedGradients(model=model, n_iter=1, baseline_fn=torch.zeros_like, create_graph=True) as attributor:
+        _, grad = attributor(data, torch.ones_like)
+    gradgrad, = torch.autograd.grad(grad.sum(), data)
+
+    assert torch.allclose(gradgrad, target), 'IntegratedGradients Attributor second order gradient mismatch!'
+
+
 def test_integrated_gradients_path(data_simple):
     '''Test whether IntegratedGradients with a single iteration and a zero-baseline is equal to the input times the
     gradient.
